//
//  NestedDiagnosticContextTest.cpp
//
//  Created  by Setsu on 6/15/10.
//  Modified by Setsu on May 30, 2013.
//  Copyright 2010-2013 RoundSquare Inc. All rights reserved.
//
// Permission is hereby granted, free of charge, to any person or organization
// obtaining a copy of the software and accompanying documentation covered by
// this license (the "Software") to use, reproduce, display, distribute,
// execute, and transmit the Software, and to prepare derivative works of the
// Software, and to permit third-parties to whom the Software is furnished to
// do so, all subject to the following:
// 
// The copyright notices in the Software and this entire statement, including
// the above license grant, this restriction and the following disclaimer,
// must be included in all copies of the Software, in whole or in part, and
// all derivative works of the Software, unless such copies or derivative
// works are solely in the form of machine-executable object code generated by
// a source language processor.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
// FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//

//----------------------------------------
//	include
//----------------------------------------
#include <Poco/NestedDiagnosticContext.h>
#include <Poco/Runnable.h>
#include <Poco/Thread.h>
#include <Poco/Format.h>

#include <sstream>

#include "ScopedLogMessage.h"
#include "PrepareConsoleLogger.h"

//----------------------------------------
//	const
//----------------------------------------
const std::size_t kNumThreads = 3;

//----------------------------------------
//	DumpNDCMessage
//----------------------------------------
void DumpNDCMessage(ScopedLogMessage& msg)
{
	std::ostringstream sstr;
	Poco::NDC::current().dump(sstr);
	msg.Message(Poco::format(	"\n- Poco::NDC::current().depth()    : %d"
								"\n- Poco::NDC::current().toString() :\n%s"
								"\n- Poco::NDC::current().dump()     :\n%s"
								, Poco::NDC::current().depth()
								, Poco::NDC::current().toString()
								, sstr.str()));
}

//----------------------------------------
//	TestFunction
//----------------------------------------
void TestFunction(ScopedLogMessage& msg, std::size_t i)
{
	poco_ndc(TestFunction);

	if(0 == i)
	{
		DumpNDCMessage(msg);
	}
	else
	{
		TestFunction(msg, --i);
	}
}

//----------------------------------------
//	MyRunnable
//----------------------------------------
class MyRunnable: public Poco::Runnable
{
public:
	MyRunnable() :
		m_pMsg(NULL)
	,	m_funcDepth(0)
	{
	}

	void setDepth(ScopedLogMessage* pMsg, std::size_t depth)
	{
		m_pMsg = pMsg;
		m_funcDepth = depth;
	}

	void run()
	{
		poco_ndc(run);

		if(NULL != m_pMsg)
		{
			TestFunction(*m_pMsg, m_funcDepth);
		}
	}

private:
	ScopedLogMessage*	m_pMsg;
	std::size_t			m_funcDepth;
};

//----------------------------------------
//	main
//----------------------------------------
int main(int /*argc*/, char** /*argv*/)
{
	PrepareConsoleLogger logger(Poco::Logger::ROOT, Poco::Message::PRIO_INFORMATION);

	ScopedLogMessage msg("NestedDiagnosticContextTest ", "start", "end");

	poco_ndc(main);

	Poco::Thread thread[kNumThreads];
	MyRunnable r[kNumThreads];
	for(std::size_t i=0; i<kNumThreads; ++i)
	{
		r[i].setDepth(&msg, i);
		thread[i].start(r[i]);
	}
	Poco::Thread::sleep(200);
	for(std::size_t i=0; i<kNumThreads; ++i)
	{
		thread[i].join();
	}

	DumpNDCMessage(msg);

	return 0;
}
